-- Would like to say thanks to the following authors for their inspiration
-- tekkub, cael, roth, and luzz 

local _, shNameplates = ...

local CreateFrame = CreateFrame
local type, error = type, error
shNameplates.eventFrame = CreateFrame("Frame", nil, UIParent)
shNameplates.eventFrame:SetScript("OnEvent", function(self, event, ...)
	if type(self[event] == "function") then
		return self[event](self, event, ...)
	end
end)

local diag = [=[Interface\AddOns\shNameplates\media\diag]=]
local cfg = shNameplates.cfg -->Get the config data

-->failsafe
if cfg.showhpvalue == false and cfg.hpallthetime == true then
	error("|cff649DDFshNameplates:|r |cffFF0000ERROR|r with config settings - hpallthetime cannot be true if showhpvalue is false!  Values reset to false.")
	cfg.showhpvalue = false
	cfg.hpallthetime = false
end

-->CVar settings to the way I like it (old school)
local SetCVar = SetCVar
shNameplates.eventFrame:RegisterEvent("ADDON_LOADED")
function shNameplates.eventFrame:ADDON_LOADED(event, addon)
	if addon and addon == "shNameplates" then
		SetCVar("bloattest", 0) 					-- 1 might make nameplates larger but it fixes the disappearing ones.
		SetCVar("bloatnameplates", 0) 				-- 1 makes nameplates larger depending on threat percentage.
		SetCVar("bloatthreat", 0) 					-- 1 makes nameplates resize depending on threat gain/loss. Only active when a mob has multiple units on its threat table.
		SetCVar("ShowClassColorInNameplate", 1)		-- Always shows class colored nameplates
	end
end

if cfg.autohide then
	shNameplates.eventFrame:RegisterEvent("PLAYER_REGEN_ENABLED")
	function shNameplates.eventFrame:PLAYER_REGEN_ENABLED()
		SetCVar("nameplateShowEnemies", 0)
	end
end

if cfg.autoshow then
	shNameplates.eventFrame:RegisterEvent("PLAYER_REGEN_DISABLED")
	function shNameplates.eventFrame:PLAYER_REGEN_DISABLED()
		SetCVar("nameplateShowEnemies", 1)
	end
end

--> Are you local?
local floor, modf, len, lower, gsub, select, upper, sub, find, SetFormattedText = math.floor, math.modf, string.len, string.lower, string.gsub, select, string.upper, string.sub, string.find, SetFormattedText
local unpack, UnitIsPVP, UnitIsEnemy, IsShown, SetSize, ClearAllPoints, GetMinMaxValues = unpack, UnitIsPVP, UnitIsEnemy, IsShown, SetSize, ClearAllPoints, GetMinMaxValues
local cFont = [=[Interface\AddOns\shNameplates\media\bullets.ttf]=]

---------------------------
-----UTILITY FUNCTIONS-----
---------------------------
local tonumber = tonumber
local format = format

local function round(num, idp)
	return tonumber(format("%." .. (idp or 0) .. "f", num))
end

local function formatNumber(number)
	if number >= 1e6 then
		return round(number/1e6, 1).."|cffEEEE00m|r"
	elseif number >= 1e3 then
		return round(number/1e3, 1).."|cffEEEE00k|r"
	else
		return number
	end
end

local function nameColoring(self, checker)
	if checker then
		if UnitIsPVP("target") and UnitIsEnemy("target", "player") then
			return self.healthBar:GetStatusBarColor()
		end
	end
	return unpack(cfg.name.color)
end

-->IsTargetFrame?
local UnitExists = UnitExists
local function IsTargetNameplate(self)
	return self:IsShown() and self:GetAlpha() >= 0.99 and UnitExists("target") or false
end

-->IsMouseover??
local function IsMouseoverNameplate(self)
	if self.highlight then
		return self.highlight:IsShown() == 1 and true or false
	end
end

-->TEKKUB's color gradient function of awesomeness
local function ColorGradient(perc, r1, g1, b1, r2, g2, b2, r3, g3, b3)
	if perc >= 1 then return r3, g3, b3 elseif perc <= 0 then return r1, g1, b1 end
	local segment, relperc = modf(perc*2)
	if segment == 1 then r1, g1, b1, r2, g2, b2 = r2, g2, b2, r3, g3, b3 end
	return r1 + (r2-r1)*relperc, g1 + (g2-g1)*relperc, b1 + (b2-b1)*relperc
end

----------------------------------
-----Castbar Helper Functions-----
----------------------------------

--> Castbar display of time
local UnitCastingInfo, UnitChannelInfo = UnitCastingInfo, UnitChannelInfo
local function UpdateTime(self, curValue)
	local minValue, maxValue = self:GetMinMaxValues()
	local chk = false
	if maxValue > 300 or maxValue == nil then chk = true end
	
	local oldname = self.channeling or self.casting
	local castname = oldname and (len(oldname) > 20) and gsub(oldname, "%s?(.[\128-\191]*)%S+%s", "%1. ") or oldname -->fixes really long names
	
	if self.channeling then
		if chk then self.time:SetFormattedText("|cffFFFFFF%.1f|r - |cff9D9D9D??|r", curValue)
		else self.time:SetFormattedText("|cffFFFFFF%.1f|r - |cff9D9D9D%.2f|r", curValue, maxValue) end
	else 
		if chk then self.time:SetFormattedText("|cffFFFFFF%.1f|r - |cff9D9D9D??|r", maxValue - curValue)
		else self.time:SetFormattedText("|cffFFFFFF%.1f|r - |cff9D9D9D%.2f|r", maxValue - curValue, maxValue) end		
	end	
	
	self.cname:SetText(castname)
end

-->Needed to fix castbar colors and bloat
local function FixCastbar(self)
	self:ClearAllPoints()
	self:SetParent(self.healthBar)
	self.castbarOverlay:Hide()	
	self:SetSize(cfg.castbar.width, cfg.castbar.height)
	self:SetPoint(cfg.castbar.anchor, self.healthBar, cfg.castbar.anchor2, cfg.castbar.xoffset, cfg.castbar.yoffset)

	if cfg.powerbar and self.powerBar:IsShown() == 1 then
		self:SetPoint(cfg.castbar.anchor, self.powerBar, cfg.castbar.anchor2, cfg.castbar.xoffset, cfg.castbar.yoffset)
	end
end

-->Color castbar depending on interruptability
local function ColorCastbar(self, shielded)
	if shielded then 
		self:SetStatusBarTexture(diag)
		self:SetStatusBarColor(0.83, 0.14, 0.14)
		self.cbBackground:SetVertexColor(0.13, 0.02, 0.02)
		self.interrupt:SetText("|cffD94426>|r")
	else
		self:SetStatusBarTexture(cfg.bartex)
		self:SetStatusBarColor(0.86, 0.71, 0.18)
		self.cbBackground:SetVertexColor(0.13, 0.11, 0.03)
		self.interrupt:SetText("|cff5CD926>|r")
	end
end

--------------------------
--- SHOW POWER UPDATE ----
--------------------------
local UnitPower, UnitPowerMax, UnitPowerType, UnitIsPlayer = UnitPower, UnitPowerMax, UnitPowerType, UnitIsPlayer
local function updatePower(self)
	local curPower = UnitPower("target")
	local maxPower = UnitPowerMax("target")
	local pType = UnitPowerType("target")
	local mType = UnitIsPlayer("target")
		
	if mType ~= 1 or ((pType == 1 or 6) and curPower <= 0) then return end
		
	if pType == 0 then 
		self.powerBar:SetStatusBarColor(unpack(cfg.powercolors.mana))
	elseif pType == 1 then
		self.powerBar:SetStatusBarColor(unpack(cfg.powercolors.rage))
	elseif pType == 2 then
		self.powerBar:SetStatusBarColor(unpack(cfg.powercolors.focus))
	elseif pType == 3 then
		self.powerBar:SetStatusBarColor(unpack(cfg.powercolors.energy))
	elseif pType == 6 then
		self.powerBar:SetStatusBarColor(unpack(cfg.powercolors.runic))
	else
		self.powerBar:SetStatusBarColor(0.55, 0.57, 0.61) -->generic color 
	end
	
	self.powerBar:SetValue(curPower)
	self.powerBar:SetMinMaxValues(0, maxPower)
	self.powerBar:Show()
end

--------------------------
--- SHOW COMBO POINT -----
--------------------------
local GetComboPoints, UnitClass = GetComboPoints, UnitClass
local class = UnitClass("player")
local function updateExtras(self)
	if not UnitExists("target") then 
		if cfg.powerbar then self.powerBar:Hide() end
		self.combo:SetText(" ")
		return
	end
			
	if IsTargetNameplate(self) then 	
		if cfg.powerbar then updatePower(self) end --> updated power bar as well!
		if class and (class == "Rogue" or class == "Druid") and self.combo then
			local points = GetComboPoints("player", "target")
			if points == 1 then 
				self.combo:SetText("|cffD94426>|r")
			elseif points == 2 then
				self.combo:SetText("|cffD94426>|r   |cffD98826>|r")
			elseif points == 3 then
				self.combo:SetText("|cffD94426>|r   |cffD98826>|r   |cffD9D326>|r")
			elseif points == 4 then
				self.combo:SetText("|cffD94426>|r   |cffD98826>|r   |cffD9D326>|r   |cffB5D926>|r")
			elseif points == 5 then
				self.combo:SetText("|cff5CD926>|r   |cff5CD926>|r   |cff5CD926>|r   |cff5CD926>|r   |cff5CD926>|r")
			else
				self.combo:SetText(" ") 
			end
		end
	else
		if cfg.powerbar then self.powerBar:Hide() end 
		self.combo:SetText(" ")
	end
end

--------------------------
--- SHOW HEALTH UPDATE ---
--------------------------
local function updateHealth(healthBar, value)
	if healthBar then
		local self = healthBar:GetParent()
		local min, maxhealth = self.healthBar:GetMinMaxValues()
		if value == "x" then 
			value = maxhealth
		end
		local currentValue = value or self.healthBar:GetValue()
		local p = (currentValue/maxhealth)*100
		local r, g, b = ColorGradient(currentValue/maxhealth, 1,0,0, 1,1,0, 0,1,0)				
		self.hp:SetTextColor(r, g, b)
		if cfg.showhpvalue then
			if p < 100 then
				self.hp:SetFormattedText("|cffFFFFFF%s|r |cffffffff:|r %.1f%%", formatNumber(currentValue), p)
			elseif p == 100 and cfg.hpallthetime then
				self.hp:SetFormattedText("|cffFFFFFF%s|r |cffffffff:|r %.0f%%", formatNumber(currentValue), p)	
			else
				self.hp:SetText("")
			end
		end
	end
end

----------------------------
--- ONUPDATE/THREAT 2in1 ---
----------------------------
local function ThreatUpdate(self, elapsed)
	self.elapsed = self.elapsed + elapsed
	
	if self.elapsed >= 0.33 then
		
		self.elapsed = 0

		--> for mouse hover and target display options
		if IsTargetNameplate(self) or IsMouseoverNameplate(self) then --im or it then 
			self.name:SetTextColor(0.85, 0.85, 0.85)
			self.name:SetShadowOffset(1, -1)
			self.name:SetShadowColor(0.3, 0.3, 0.3, 0.9)
			self.healthBar.hpGlow:SetBackdropBorderColor(0.65, 0.65, 0.65)  
			self.healthBar.hpGlow:SetAlpha(0.8)
		else
			self.name:SetTextColor(unpack(cfg.name.color))
			self.name:SetFont(cfg.name.font, cfg.name.fontSize, cfg.name.fontFlag) 
			self.name:SetShadowColor(0, 0, 0, 0)
			self.healthBar.hpGlow:SetBackdropBorderColor(0.4, 0.4, 0.4)
			self.healthBar.hpGlow:SetAlpha(0.5)
		end	
		
		--> added to show combo points and power bar
		updateExtras(self)  
				
		-->used for threat comparisons and updates to coloring
		local r, g, b = self.oldglow:GetVertexColor()
		
		-->LOW or no threat
		if self.oldglow:IsShown() ~= 1 then
			self.healthBar:SetStatusBarColor(self.r, self.g, self.b)
		-->AGGRO
		elseif g + b == 0 then
			self.healthBar.hpGlow:SetBackdropBorderColor(1, 0, 0)	
			if cfg.tankmode then 
				self.healthBar:SetStatusBarColor(unpack(cfg.colors.aggro)) 
			else 
				self.healthBar:SetStatusBarColor(self.r, self.g, self.b)
			end		
		-->HIGHTHREAT, but not aggro...yet
		elseif r + g > 0 then
			self.healthBar.hpGlow:SetBackdropBorderColor(1, 1, 0)
			if cfg.tankmode then
				self.healthBar:SetStatusBarColor(unpack(cfg.colors.highthreat))
			else
				self.healthBar:SetStatusBarColor(self.r, self.g, self.b)
			end
		end			
	end
end

--------------------
--- UPDATE PLATE ---
--------------------
local UnitLevel = UnitLevel
local function UpdatePlate(self)
	local r, g, b = self.healthBar:GetStatusBarColor()
	local newr, newg, newb
	if g + b == 0 then
		-- Hostile unit
		newr, newg, newb = unpack(cfg.colors.hostile) 
	elseif r + b == 0 then
		-- Friendly unit
		newr, newg, newb = unpack(cfg.colors.friendlynpc) 
	elseif r + g == 0 then
		-- Friendly player
		newr, newg, newb = unpack(cfg.colors.friendly) 
	elseif 2 - (r + g) < 0.05 and b == 0 then
		-- Neutral unit
		newr, newg, newb = unpack(cfg.colors.neutral)
	else
		-- Hostile player - class colored.
		newr, newg, newb = r, g, b
	end
	
	self.healthBar:SetStatusBarColor(newr, newg, newb)
	self.r, self.g, self.b = newr, newg, newb
	
	self.healthBar:ClearAllPoints()
	self.healthBar:SetPoint("CENTER", self.healthBar:GetParent(), 0, cfg.healthbar.yoffset)
	self.healthBar:SetSize(cfg.healthbar.width, cfg.healthbar.height)
	self.healthBar.hpBackground:SetVertexColor(0.15, 0.15, 0.15, 0.8)
	
	-->initial castbar maintenance
	if self.castBar:IsShown() then self.castBar:Hide() end
	self.castBar.IconOverlay:SetVertexColor(0.6, 0.6, 0.6)
		
	self.highlight:ClearAllPoints()
	self.highlight:SetAllPoints(self.healthBar)
		
	local oldName = self.oldname:GetText()
	local newName = (len(oldName) > 30) and gsub(oldName, "%s?(.[\128-\191]*)%S+%s", "%1. ") or oldName -->fixes really long names
	self.name:SetTextColor(nameColoring(self, cfg.namecolor))
	if not cfg.name.uppercase then 
		self.name:SetText(lower(newName)) 
	else 
		self.name:SetText(newName) 
	end
		
	local level, elite, rare, mylevel = tonumber(self.level:GetText()), self.elite:IsShown(), UnitLevel("player")
	self.level:ClearAllPoints()
	self.level:SetPoint(cfg.level.anchor, self.healthBar, cfg.level.anchor2, cfg.level.xoffset, cfg.level.yoffset)
	if self.boss:IsShown() then 
		self.mobtype:SetTexture([=[Interface\AddOns\shNameplates\media\boss]=])
	elseif elite and not self.elite:GetTexture() == "Interface\\Tooltips\\EliteNameplateIcon" then
		self.level:SetText("(R)"..level)
		self.mobtype:SetTexture([=[Interface\AddOns\shNameplates\media\elite]=])
	elseif elite then
		self.level:SetText(level)
		self.mobtype:SetTexture([=[Interface\AddOns\shNameplates\media\elite]=])
	else
		self.mobtype:SetTexture([=[Interface\AddOns\shNameplates\media\regular]=] or nil)
	end	
	
	updateExtras(self)
	
	self.fade:SetChange(self:GetAlpha())
	self:SetAlpha(0)
	self.ag:Play()
end

----------------------------
--- EVENT HANDLERS/CUSTOM---
----------------------------
local function OnSizeChanged(self, width, height)
	if self:IsShown() ~= 1 then return end
		
	if height > cfg.castbar.height then
		self.needFix = true
	end
end

local function OnValueChanged(self, curValue)
	if self:IsShown() ~= 1 then return end
	UpdateTime(self, curValue) 
	
	-->fix castbar from bloating - as a back up to onshow fixcastbar call
	if self:GetHeight() > cfg.castbar.height or self.needFix then
		FixCastbar(self)
		self.needFix = nil
	end
	
	local _, g, _, _ = self:GetStatusBarColor()
	if self.controller and g > 0.15 then 
		self:SetStatusBarColor(0.83, 0.14, 0.14) 
	end
end

local function OnShow(self)	
	FixCastbar(self)
	self.IconOverlay:Show()
	ColorCastbar(self, self.shieldedRegion:IsShown() == 1) 
end

local function CastbarEvents(self, event, unit)
	if unit == "target" then
		local chc
		local cc
		self.controller = nil
		self.channeling, _, _, _, _, _, _, chc = UnitChannelInfo("target")
		self.casting, _, _, _, _, _, _, _, cc = UnitCastingInfo("target")

		if self.channeling and not self.casting then self.controller = chc
		else self.controller = cc end
		
		if self:IsShown() == 1 then
			ColorCastbar(self, event == "UNIT_SPELLCAST_NOT_INTERRUPTIBLE" or self.controller or self.shieldedRegion:IsShown() == 1) 
		end
	end
end

--> Event used to update powerbar and combo points on current target
local function FrameEvents(self, event, unit)
	if event == "PLAYER_TARGET_CHANGED" then 
		updateExtras(self)
	elseif event == "UNIT_POWER" and cfg.power and unit == "target" then
		updateExtras(self)
	end
end

--------------------
--- CREATE PLATE ---
--------------------
local function SkinPlate(frame)
	if frame.done then
		return
	end

	frame.nameplate = true

	frame.healthBar, frame.castBar = frame:GetChildren()
	local healthBar, castBar = frame.healthBar, frame.castBar
	local glowRegion, overlayRegion, highlightRegion, nameTextRegion, levelTextRegion, bossIconRegion, raidIconRegion, stateIconRegion = frame:GetRegions()
	local _, castbarOverlay, shieldedRegion, spellIconRegion= castBar:GetRegions()
	
	-->hide uglies
	glowRegion:SetTexture("")	
    overlayRegion:SetTexture("")	
    shieldedRegion:SetTexture("")	
	castbarOverlay:SetTexture("")	
    stateIconRegion:SetTexture("")	
    bossIconRegion:SetTexture("")

	frame.oldname = nameTextRegion
	nameTextRegion:Hide()
		
	------------------
	---NAME TEXT------
	------------------
	local newNameRegion = frame:CreateFontString(nil, 'OVERLAY')
	newNameRegion:SetParent(healthBar)
	newNameRegion:SetPoint(cfg.name.anchor, healthBar, cfg.name.anchor2, cfg.name.xoffset, cfg.name.yoffset)
	newNameRegion:SetFont(cfg.name.font, cfg.name.fontSize, cfg.name.fontFlag) 
	frame.name = newNameRegion
	
	------------------
	---HEALTH TEXT----
	------------------
	if cfg.showhpvalue then
		local hpRegion = frame:CreateFontString(nil, 'OVERLAY')
		hpRegion:SetPoint(cfg.health.anchor, healthBar, cfg.health.anchor2, cfg.health.xoffset, cfg.health.yoffset)
		hpRegion:SetFont(cfg.health.font, cfg.health.fontSize, cfg.health.fontFlag)
		frame.hp = hpRegion
	end
	
	-----------------------
	---COMBO POINT ICONS---
	-----------------------
	local comboRegion = frame:CreateFontString(nil, 'OVERLAY')
	comboRegion:SetPoint("TOPRIGHT", healthBar, "BOTTOMRIGHT", 0, 3)
	comboRegion:SetFont(cFont, 6, "THINOUTLINE")
	frame.combo = comboRegion
	
	-----------------------
	---LEVEL TEXT INFO ----
	-----------------------
	levelTextRegion:SetFont(cfg.level.font, cfg.level.fontSize, cfg.level.fontFlag)
	levelTextRegion:SetShadowOffset(0.5, -0.5)
	levelTextRegion:SetShadowColor(0, 0, 0, 0.9)
	frame.level = levelTextRegion
	
	---------------------
	---HEALTHBAR stuff---
	---------------------
	healthBar:SetStatusBarTexture(cfg.bartex)
	healthBar:SetFrameLevel(5)
	
	healthBar.hpBackground = healthBar:CreateTexture(nil, "BACKGROUND")
	healthBar.hpBackground:SetAllPoints()
	healthBar.hpBackground:SetTexture(cfg.bartex)
	healthBar.hpBackground:SetVertexColor(0.15, 0.15, 0.15, 0.8)
			
	healthBar.hpGlow = CreateFrame("Frame", nil, healthBar)
	healthBar.hpGlow:SetFrameLevel(healthBar:GetFrameLevel() -1 > 0 and healthBar:GetFrameLevel() -1 or 0)
	healthBar.hpGlow:SetPoint("TOPLEFT", healthBar, "TOPLEFT", -3, 3)
	healthBar.hpGlow:SetPoint("BOTTOMRIGHT", healthBar, "BOTTOMRIGHT", 3, -3)
	healthBar.hpGlow:SetBackdrop(cfg.backdrop)
	healthBar.hpGlow:SetAlpha(0.5)
	healthBar.hpGlow:SetBackdropColor(0, 0, 0, 0)
	healthBar.hpGlow:SetBackdropBorderColor(0.4, 0.4, 0.4)
	
	-->hp event script options
	if cfg.showhpvalue then
		healthBar:SetScript("OnValueChanged", updateHealth)
		if cfg.hpallthetime then 
			updateHealth(healthBar, 'x')
		end
	end

	-------------------------
	---CASTBAR ATTRIBUTES----
	-------------------------
	castBar.castbarOverlay = castbarOverlay
	castBar.shieldedRegion = shieldedRegion
	castBar.healthBar = healthBar
	castBar:SetStatusBarTexture(cfg.bartex)
	castBar:SetParent(healthBar)
	castBar:SetFrameLevel(1) -- > 0 or 0)
	castBar:ClearAllPoints()
	castBar:SetPoint(cfg.castbar.anchor, healthBar, cfg.castbar.anchor2, cfg.castbar.xoffset, cfg.castbar.yoffset)
	castBar:SetSize(cfg.castbar.width, cfg.castbar.height)	

	castBar:HookScript("OnShow", OnShow)
	castBar:SetScript("OnValueChanged", OnValueChanged)
	castBar:SetScript("OnSizeChanged", OnSizeChanged)
	castBar:SetScript("OnEvent", CastbarEvents)
	castBar:RegisterEvent("UNIT_SPELLCAST_INTERRUPTIBLE")
	castBar:RegisterEvent("UNIT_SPELLCAST_NOT_INTERRUPTIBLE")
	castBar:RegisterEvent("UNIT_SPELLCAST_CHANNEL_START")
	castBar:RegisterEvent("UNIT_SPELLCAST_START")
			
	castBar.time = castBar:CreateFontString(nil, "ARTWORK")
	castBar.time:SetPoint(cfg.casttime.anchor, castBar, cfg.casttime.anchor2, cfg.casttime.xoffset, cfg.casttime.yoffset)
	castBar.time:SetFont(cfg.casttime.font, cfg.casttime.fontSize, cfg.casttime.fontFlag)
	castBar.time:SetTextColor(0.95, 0.95, 0.95)
	castBar.time:SetShadowOffset(1, -1)
	castBar.time:SetShadowColor(0.3, 0.3, 0.3, 0.9)
	
	castBar.cname = castBar:CreateFontString(nil, "ARTWORK")
	castBar.cname:SetPoint(cfg.spellname.anchor, castBar, cfg.spellname.anchor2, cfg.spellname.xoffset, cfg.spellname.yoffset)
	castBar.cname:SetFont(cfg.spellname.font, cfg.spellname.fontSize, cfg.spellname.fontFlag)
	castBar.cname:SetTextColor(0.95, 0.95, 0.95)
	castBar.cname:SetShadowOffset(1, -1)
	castBar.cname:SetShadowColor(0.3, 0.3, 0.3, 0.9)
	
	castBar.cbBackground = castBar:CreateTexture(nil, "BACKGROUND")
	castBar.cbBackground:SetAllPoints()
	castBar.cbBackground:SetTexture(cfg.bartex)
	castBar.cbBackground:SetVertexColor(0.2, 0.2, 0.2, 0.9)

	castBar.cbGlow = CreateFrame("Frame", nil, castBar)
	castBar.cbGlow:SetFrameLevel(castBar:GetFrameLevel() -1 > 0 and castBar:GetFrameLevel() -1 or 0)
	castBar.cbGlow:SetPoint("TOPLEFT", castBar, -3, 3)
	castBar.cbGlow:SetPoint("BOTTOMRIGHT", castBar, 3, -3)
	castBar.cbGlow:SetBackdrop(cfg.backdrop)
	castBar.cbGlow:SetBackdropColor(0, 0, 0, 0)
	castBar.cbGlow:SetBackdropBorderColor(0.5, 0.5, 0.5, 0.8)

	castBar.HolderA = CreateFrame("Frame", nil, castBar)
	castBar.HolderA:SetFrameLevel(castBar.HolderA:GetFrameLevel() + 1)
	castBar.HolderA:SetAllPoints()

	castBar.spellicon = spellIconRegion
	castBar.spellicon:ClearAllPoints()
	castBar.spellicon:SetParent(castBar)
	castBar.spellicon:SetPoint(cfg.spellicon.anchor, castBar, cfg.spellicon.anchor2, cfg.spellicon.xoffset, cfg.spellicon.yoffset)
	castBar.spellicon:SetSize(cfg.spellicon.size, cfg.spellicon.size)
	
	castBar.HolderB = CreateFrame("Frame", nil, castBar)
	castBar.HolderB:SetFrameLevel(castBar.HolderA:GetFrameLevel() + 2)
	castBar.HolderB:SetAllPoints()

	castBar.IconOverlay = castBar.HolderB:CreateTexture(nil, "OVERLAY")
	castBar.IconOverlay:SetPoint("TOPLEFT", spellIconRegion, -1.5, 1.5)
	castBar.IconOverlay:SetPoint("BOTTOMRIGHT", spellIconRegion, 1.5, -1.5)
	castBar.IconOverlay:SetTexture(cfg.icontex)
	
	-------------------------
	---Interruptable icon----
	-------------------------
	castBar.interrupt = castBar:CreateFontString(nil, "OVERLAY", castBar)
	castBar.interrupt:SetPoint("CENTER", castBar, "BOTTOM")
	castBar.interrupt:SetFont(cFont, 6, "OUTLINE")
	
	---------------------
	---POWER/MANA Bar----
	---------------------
	if cfg.powerbar then
		local powerBar = CreateFrame("StatusBar", nil)
		powerBar:SetParent(healthBar)
		powerBar:SetFrameLevel(2)
		powerBar:SetStatusBarTexture(cfg.bartex)
		powerBar:SetPoint("TOP", healthBar, "BOTTOM", 0, -1.5)
		powerBar:SetSize(cfg.healthbar.width, cfg.healthbar.height * 0.5)
				
		powerBar.pbBackground = powerBar:CreateTexture(nil, "BACKGROUND")
		powerBar.pbBackground:SetAllPoints()
		powerBar.pbBackground:SetTexture(cfg.bartex)
		powerBar.pbBackground:SetVertexColor(0.1, 0.1, 0.1, 0.9)
	
		powerBar.pGlow = CreateFrame("Frame", nil, powerBar)
		powerBar.pGlow:SetPoint("TOPLEFT", powerBar, "TOPLEFT", -3, 3)
		powerBar.pGlow:SetPoint("BOTTOMRIGHT", powerBar, "BOTTOMRIGHT", 3, -3)
		powerBar.pGlow:SetBackdrop(cfg.backdrop)
		powerBar.pGlow:SetFrameLevel(powerBar:GetFrameLevel() -1 > 0 and powerBar:GetFrameLevel() -1 or 0)
		powerBar.pGlow:SetBackdropColor(0, 0, 0, 0)
		powerBar.pGlow:SetBackdropBorderColor(0.65, 0.65, 0.65, 0.8)
		powerBar:Hide()
		frame.powerBar = powerBar
		castBar.powerBar = powerBar
	end
	
	-----------------------
	---HIGHTLIGHT REGION---
	-----------------------
	highlightRegion:SetTexture(cfg.bartex)
	highlightRegion:SetVertexColor(0.25, 0.25, 0.25, 0.8)
	frame.highlight = highlightRegion

	---------------------
	---RAID ICON-----
	---------------------
	raidIconRegion:ClearAllPoints()
	raidIconRegion:SetPoint(cfg.raidicon.anchor, healthBar, cfg.raidicon.anchor2, cfg.raidicon.xoffset, cfg.raidicon.yoffset)
	raidIconRegion:SetSize(cfg.raidicon.size, cfg.raidicon.size)
	raidIconRegion:SetTexture(cfg.raidicon.textures)

	---------------------
	---ELITE/BOSS ICON-----
	---------------------
	local mobtype = healthBar:CreateTexture(nil, "OVERLAY")
	mobtype:SetPoint("CENTER", healthBar, "TOPLEFT", -1, 1)
	mobtype:SetSize(22,22)
	mobtype:SetAlpha(0.6)
	frame.mobtype = mobtype
	
	frame.oldglow = glowRegion
	frame.elite = stateIconRegion
	frame.boss = bossIconRegion
	
	-- animations for initial fade in
	frame.ag = frame:CreateAnimationGroup()
	frame.fade = frame.ag:CreateAnimation('Alpha')
	frame.fade:SetSmoothing("OUT")
	frame.fade:SetDuration(0.5)
	frame.fade:SetChange(1)
	frame.ag:SetScript('OnFinished', function()
		frame:SetAlpha(frame.fade:GetChange())
		-- otherwise it flashes
	end)
	
	---------------------
	---EVENT SCRIPTS-----
	---------------------
	UpdatePlate(frame)
	frame:RegisterEvent("UNIT_POWER")
	frame:RegisterEvent("PLAYER_TARGET_CHANGED")
	frame:SetScript("OnEvent", FrameEvents)
	frame:HookScript("OnShow", UpdatePlate) 
	frame:HookScript("OnUpdate", ThreatUpdate)
	
	frame.done = true -->used for iterator function
	frame.elapsed = 0
end

-----------------------------
-----CREATE ALL PLATES-------
---- thanks to Rothar -------
-----------------------------
-->Check to see if frame is a nameplate and if so, create it once
local function IsNamePlateFrame(frame)
	local o = select(2, frame:GetRegions())
    if not o or o:GetObjectType() ~= "Texture" or o:GetTexture() ~= "Interface\\Tooltips\\Nameplate-Border" then 
		frame.done = true --don't touch this frame again
		return false 
    end    
	return true
end

-->Iterate over nameplate frames and create them - only once!
local lastupdate = 0
local i = 0
local function iterateNameplates(self,elapsed)
	lastupdate = lastupdate + elapsed
    if lastupdate > 0.33 then
		lastupdate = 0
		local num = select("#", WorldFrame:GetChildren())
		for i = 1, num do
			local frame = select(i, WorldFrame:GetChildren())      
			if not frame.done and IsNamePlateFrame(frame) then 
				SkinPlate(frame)
			end
		end 
    end
end

-->Register initial login event and set onupdate function
local updateFrame = CreateFrame("Frame")
updateFrame:SetScript("OnEvent", function(self, event)
	if(event=="PLAYER_LOGIN") then
		self:SetScript("OnUpdate", iterateNameplates)
    end
end)
updateFrame:RegisterEvent("PLAYER_LOGIN")
--END OF ADDON--  